﻿
''' <summary>
''' Ein Event, welches durch einen Timer verzögert ausgelöst wird.
''' </summary>
Public Class DelayedEvent

    ''' <summary>
    ''' Wird ausgelöst, sobald die Zeitverzögerung abgelaufen ist.
    ''' </summary>
    Public Event Elapsed()

    Dim _DelayMilliseconds As Integer
    ''' <summary>
    ''' Gibt die Zeitverzögerung in Millisekunden an.
    ''' </summary>
    Public Property DelayMilliseconds As Integer
        <DebuggerStepThrough()> _
        Get
            Return _DelayMilliseconds
        End Get
        <DebuggerStepThrough()> _
        Set(value As Integer)
            If value < 0 Then
                Throw New ArgumentOutOfRangeException("value", value, "Der Wert darf nicht negativ sein.")
            End If
            If Not _DelayMilliseconds = value Then
                _DelayMilliseconds = value
                If _IsWaiting Then
                    DelayTimer.Interval = _DelayMilliseconds
                    DelayTimer.Stop()
                    DelayTimer.Start()
                End If
            End If
        End Set
    End Property

    ''' <summary>
    ''' Ein Control, das verwendet wird, um das Auslösen des Events im GUI-Thread durchzuführen.
    ''' Ist diese Eigenschaft Nothing, wird das Event in einem anderen Thread ausgelöst.
    ''' </summary>
    Public Property DelegateControl As System.Windows.Forms.Control

    Dim _IsWaiting As Boolean = False
    ''' <summary>
    ''' Gibt an, ob die Zeitspanne gerade abläuft.
    ''' </summary>
    Public ReadOnly Property IsWaiting As Boolean
        <DebuggerStepThrough()> _
        Get
            Return _IsWaiting
        End Get
    End Property

    Dim DelayTimer As Timers.Timer

    <DebuggerStepThrough()> _
    Public Sub New(NewDelayMilliSeconds As Integer)
        _DelayMilliseconds = NewDelayMilliSeconds
        DelayTimer = New Timers.Timer With {.AutoReset = False, .Enabled = False, .Interval = _DelayMilliseconds}
        AddHandler DelayTimer.Elapsed, AddressOf TimerElapsed
    End Sub

    <DebuggerStepThrough()> _
    Public Sub New(NewDelayMilliSeconds As Integer, NewDelegateControl As System.Windows.Forms.Control)
        Me.New(NewDelayMilliSeconds)
        _DelegateControl = NewDelegateControl
    End Sub

    ''' <summary>
    ''' Beginnt das Abwarten der Zeitverzögerung.
    ''' Ist die Zeitverzögerung seit dem letzten Aufruf noch nicht abgelaufen, wird sie zurückgesetzt.
    ''' </summary>
    <DebuggerStepThrough()> _
    Public Sub Trigger()
        If _IsWaiting Then
            DelayTimer.Stop()
            DelayTimer.Start()
        Else
            DelayTimer.Start()
            _IsWaiting = True
        End If
    End Sub

    ''' <summary>
    ''' Bricht das Abwarten der Zeitverzögerung ab.
    ''' </summary>
    <DebuggerStepThrough()> _
    Public Sub Cancel()
        If _IsWaiting Then
            DelayTimer.Stop()
            _IsWaiting = False
        End If
    End Sub

    <DebuggerStepThrough()> _
    Private Sub TimerElapsed(sender As Object, e As Timers.ElapsedEventArgs)
        _IsWaiting = False
        If _DelegateControl Is Nothing Then
            OnElapsed()
        Else
            _DelegateControl.BeginInvoke(Sub() OnElapsed())
        End If
    End Sub

    <DebuggerStepThrough()> _
    Private Sub OnElapsed()
        RaiseEvent Elapsed()
    End Sub

End Class
