﻿
Imports System.Reflection
Imports System.Reflection.Emit

Imports OnUtils.Extensions

Namespace Emit

    ''' <summary>
    ''' Bietet eine einfache Möglichkeit, Assemblies zu definieren.
    ''' </summary>
    Public Class EmitAssembly

        Dim _Assembly As AssemblyBuilder
        ''' <summary>
        ''' Die <see cref="AssemblyBuilder"/>-Instanz, in der das Modul <see cref="MainModule"/> definiert ist.
        ''' </summary>
        Public ReadOnly Property Assembly As AssemblyBuilder
            <DebuggerStepThrough()> _
            Get
                Return _Assembly
            End Get
        End Property

        Dim _MainModule As ModuleBuilder
        ''' <summary>
        ''' Das Modul, in dem Typen definiert werden.
        ''' </summary>
        Public ReadOnly Property MainModule As ModuleBuilder
            <DebuggerStepThrough()> _
            Get
                Return _MainModule
            End Get
        End Property

        Dim _FilePath As String
        ''' <summary>
        ''' Der vollständige Pfad zur Datei.
        ''' <example>C:\Directory\File.dll</example>
        ''' </summary>
        Public ReadOnly Property FilePath As String
            <DebuggerStepThrough()> _
            Get
                Return _FilePath
            End Get
        End Property

        Dim _DirectoryPath As String
        ''' <summary>
        ''' Der Vollständige Pfad zum Ordner, in dem sich die Datei befindet.
        ''' <example>C:\Directory</example>
        ''' </summary>
        Public ReadOnly Property DirectoryPath As String
            <DebuggerStepThrough()> _
            Get
                Return _DirectoryPath
            End Get
        End Property

        Dim _FileName As String
        ''' <summary>
        ''' Der Name der Datei mit Erweiterung, ohne Pfad zum Ordner.
        ''' <example>File.dll</example>
        ''' </summary>
        Public ReadOnly Property FileName As String
            <DebuggerStepThrough()> _
            Get
                Return _FileName
            End Get
        End Property

        Dim _FileNameWithoutExtension As String
        ''' <summary>
        ''' Der Name der Datei ohne Erweiterung und ohne Pfad zum Ordner.
        ''' <example>File</example>
        ''' </summary>
        Public ReadOnly Property FileNameWithoutExtension As String
            <DebuggerStepThrough()> _
            Get
                Return _FileNameWithoutExtension
            End Get
        End Property

        Dim _FileExtension As String
        ''' <summary>
        ''' Die Erweiterung des Dateinamens.
        ''' <example>.dll</example>
        ''' </summary>
        Public ReadOnly Property FileExtension As String
            <DebuggerStepThrough()> _
            Get
                Return _FileExtension
            End Get
        End Property

        Dim _IsExecutable As Boolean
        ''' <summary>
        ''' Gibt an, ob die Assembly eine ausführbare Datei (.exe) oder eine Klassenbibliothek (.dll) beschreibt.
        ''' Bei ausführbaren Dateien muss ein Einstiegspunkt definiert sein.
        ''' </summary>
        Public ReadOnly Property IsExecutable As Boolean
            <DebuggerStepThrough()> _
            Get
                Return _IsExecutable
            End Get
        End Property

        Dim _DefinedTypeBuilders As New List(Of TypeBuilder)
        ''' <summary>
        ''' Die in dieser Assembly definierten <see cref="TypeBuilder"/>-Objekte.
        ''' Enthält nur TypeBuilder, die mit <see cref="EmitAssembly"/>.<see cref="DefineType"/> definiert wurden.
        ''' </summary>
        Public ReadOnly Property DefinedTypeBuilders As IEnumerable(Of TypeBuilder)
            <DebuggerStepThrough()> _
            Get
                Return _DefinedTypeBuilders
            End Get
        End Property

        ''' <summary>
        ''' Konstruktor.
        ''' </summary>
        ''' <param name="NewFilePath">Der vollständige Dateipfad, an dem die Assembly abgespeichert wird.</param>
        ''' <exception cref="ArgumentException">Die Erweiterung ist weder .exe, noch .dll.</exception>
        <DebuggerStepThrough()> _
        Public Sub New(NewFilePath As String)
            _FileExtension = System.IO.Path.GetExtension(NewFilePath).ToLower
            If _FileExtension = ".exe" Then
                _IsExecutable = True
            Else
                If _FileExtension = ".dll" Then
                    _IsExecutable = False
                Else
                    Throw New ArgumentException("Der Dateipfad muss auf .dll oder .exe enden.", "NewFilePath")
                End If
            End If
            _FilePath = NewFilePath
            _FileNameWithoutExtension = System.IO.Path.GetFileNameWithoutExtension(_FilePath)
            _FileName = System.IO.Path.GetFileName(_FilePath)
            _DirectoryPath = System.IO.Path.GetDirectoryName(_FilePath)
            _Assembly = AppDomain.CurrentDomain.DefineDynamicAssembly(New AssemblyName(_FileNameWithoutExtension), AssemblyBuilderAccess.RunAndSave, _DirectoryPath)
            _MainModule = _Assembly.DefineDynamicModule(_FileName, _FileName, False)
        End Sub

        ''' <summary>
        ''' Definiert eine öffentliche Klasse <paramref name="Name"/>, die nur von Object erbt.
        ''' </summary>
        ''' <param name="Name">Der Name des neuen Types. Namespaces werden mit Punkt getrennt vorangestellt.</param>
        <DebuggerStepThrough()> _
        Public Function DefineType(Name As String) As TypeBuilder
            Return DefineType(Name, TypeAttributes.Public Or TypeAttributes.Class, GetType(Object), {})
        End Function

        ''' <summary>
        ''' Definiert einen dynamischen Typ <paramref name="Name"/>, der nur von Object erbt.
        ''' </summary>
        ''' <param name="Name">Der Name des neuen Types. Namespaces werden mit Punkt getrennt vorangestellt.</param>
        ''' <param name="Attributes">Die TypeAttributes, die den neuen Typ beschreiben.</param>
        <DebuggerStepThrough()> _
        Public Function DefineType(Name As String, Attributes As TypeAttributes) As TypeBuilder
            Return DefineType(Name, Attributes, GetType(Object), {})
        End Function

        ''' <summary>
        ''' Definiert eine öffentliche Klasse <paramref name="Name"/>, die von <paramref name="Parent"/> erbt.
        ''' </summary>
        ''' <param name="Name">Der Name des neuen Types. Namespaces werden mit Punkt getrennt vorangestellt.</param>
        ''' <param name="Parent">Der Typ, von dem der neue Typ erbt.</param>
        <DebuggerStepThrough()> _
        Public Function DefineType(Name As String, Parent As Type) As TypeBuilder
            Return DefineType(Name, TypeAttributes.Public Or TypeAttributes.Class, Parent, {})
        End Function

        ''' <summary>
        ''' Definiert einen dynamischen Typ <paramref name="Name"/>, der von <paramref name="Parent"/> erbt.
        ''' </summary>
        ''' <param name="Name">Der Name des neuen Types. Namespaces werden mit Punkt getrennt vorangestellt.</param>
        ''' <param name="Attributes">Die TypeAttributes, die den neuen Typ beschreiben.</param>
        ''' <param name="Parent">Der Typ, von dem der neue Typ erbt.</param>
        <DebuggerStepThrough()> _
        Public Function DefineType(Name As String, Attributes As TypeAttributes, Parent As Type) As TypeBuilder
            Return DefineType(Name, Attributes, Parent, {})
        End Function

        ''' <summary>
        ''' Definiert eine öffentliche Klasse <paramref name="Name"/>, die nur von Object erbt und die angegebenen Interfaces implementiert.
        ''' </summary>
        ''' <param name="Name">Der Name des neuen Types. Namespaces werden mit Punkt getrennt vorangestellt.</param>
        ''' <param name="Interfaces">Die Interfaces, die der neue Typ implementiert.</param>
        <DebuggerStepThrough()> _
        Public Function DefineType(Name As String, Interfaces As Type()) As TypeBuilder
            Return DefineType(Name, TypeAttributes.Public Or TypeAttributes.Class, GetType(Object), Interfaces)
        End Function

        ''' <summary>
        ''' Definiert einen dynamischen Typ <paramref name="Name"/>, der nur von Object erbt und die angegebenen Interfaces implementiert.
        ''' </summary>
        ''' <param name="Name">Der Name des neuen Types. Namespaces werden mit Punkt getrennt vorangestellt.</param>
        ''' <param name="Attributes">Die TypeAttributes, die den neuen Typ beschreiben.</param>
        ''' <param name="Interfaces">Die Interfaces, die der neue Typ implementiert.</param>
        <DebuggerStepThrough()> _
        Public Function DefineType(Name As String, Attributes As TypeAttributes, Interfaces As Type()) As TypeBuilder
            Return DefineType(Name, Attributes, GetType(Object), Interfaces)
        End Function

        ''' <summary>
        ''' Definiert eine öffentliche Klasse <paramref name="Name"/>, die von <paramref name="Parent"/> erbt und die angegebenen Interfaces implementiert.
        ''' </summary>
        ''' <param name="Name">Der Name des neuen Types. Namespaces werden mit Punkt getrennt vorangestellt.</param>
        ''' <param name="Parent">Der Typ, von dem der neue Typ erbt.</param>
        ''' <param name="Interfaces">Die Interfaces, die der neue Typ implementiert.</param>
        <DebuggerStepThrough()> _
        Public Function DefineType(Name As String, Parent As Type, Interfaces As Type()) As TypeBuilder
            Return DefineType(Name, TypeAttributes.Public Or TypeAttributes.Class, Parent, Interfaces)
        End Function

        ''' <summary>
        ''' Definiert einen dynamischen Typ <paramref name="Name"/>, der von <paramref name="Parent"/> erbt und die angegebenen Interfaces implementiert.
        ''' </summary>
        ''' <param name="Name">Der Name des neuen Types. Namespaces werden mit Punkt getrennt vorangestellt.</param>
        ''' <param name="Attributes">Die TypeAttributes, die den neuen Typ beschreiben.</param>
        ''' <param name="Parent">Der Typ, von dem der neue Typ erbt.</param>
        ''' <param name="Interfaces">Die Interfaces, die der neue Typ implementiert.</param>
        <DebuggerStepThrough()> _
        Public Function DefineType(Name As String, Attributes As TypeAttributes, Parent As Type, Interfaces As Type()) As TypeBuilder
            Dim TB = _MainModule.DefineType(Name, Attributes, Parent, Interfaces)
            _DefinedTypeBuilders.Add(TB)
            Return TB
        End Function

        ''' <summary>
        ''' Schließt offene TypeBuilder, die mit <see cref="EmitAssembly"/>.<see cref="DefineType"/> definiert wurden und speichert die Assembly bei <see cref="FilePath"/> ab.
        ''' </summary>
        ''' <exception cref="InvalidOperationException">Es handelt sich um eine ausführbare Datei, aber es wurde kein Einstiegspunkt festgelegt.</exception>
        <DebuggerStepThrough()> _
        Public Sub Save()
            If _IsExecutable AndAlso _Assembly.EntryPoint Is Nothing Then
                Throw New InvalidOperationException("Für die Assembly wurde kein Einstiegspunkt (Main-Methode) definiert.")
            End If
            For Each i In _DefinedTypeBuilders
                If Not i.IsCreated Then
                    i.CreateType()
                End If
            Next
            If Not System.IO.Directory.Exists(_DirectoryPath) Then
                System.IO.Directory.CreateDirectory(_DirectoryPath)
            End If
            _Assembly.Save(_FileName)
        End Sub

    End Class

End Namespace
