﻿
Imports OnUtils.Emit.IL
Imports System.Runtime.CompilerServices
Imports System.Reflection
Imports System.Reflection.Emit

Namespace Emit.Macros

    ''' <summary>
    ''' Beinhaltet Extension-Methoden für einige Klassen im <see cref="System.Reflection.Emit"/>-Namespace.
    ''' </summary>
    Public Module Macros

        ''' <summary>
        ''' Legt ein <see cref="System.Type"/>-Objekt auf dem Auswertungsstapel ab, das den angegebenen, zur Laufzeit des erstellenden Codes bekannten Type <param name="T"/> darstellt.
        ''' </summary>
        ''' <param name="T">Der zu verwendende Type.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Sub [GetType](Target As ILGenerator, T As Type)
            If T Is Nothing Then
                Throw New ArgumentNullException("T")
            End If
            Target.Ldtoken(T)
            Target.Call(PredefinedMembers.System.Type.GetTypeFromHandle_RuntimeTypeHandle)
        End Sub

        ''' <summary>
        ''' Castet das Objekt auf dem Auswertungsstapel zu einem anderen Typ durch und beachtet, ob es sich um einen Referenz- oder Wertetyp handelt.
        ''' Entspricht DirectCast() in Visual Basic und dem as-Operator in C#.
        ''' Bei Object wird ein Aufruf an <see cref="RuntimeHelpers"/>.GetObjectValue(Object) verwendet.
        ''' Bei Wertetypen wird der OpCode unbox_any verwendet.
        ''' Bei Referenztypen wird der OpCode castclass verwendet.
        ''' </summary>
        ''' <param name="TargetType">Der Typ, zu den gecastet werden soll.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Sub [DirectCast](Target As ILGenerator, TargetType As Type)
            If TargetType Is Nothing Then
                Throw New ArgumentNullException("TargetType")
            ElseIf TargetType Is GetType(Object) Then
                Target.Call(PredefinedMembers.System.Runtime.CompilerServices.RuntimeHelpers.GetObjectValue_Object)
            ElseIf TargetType.IsValueType Then
                Target.Unbox_Any(TargetType)
            ElseIf TargetType.IsInterface OrElse TargetType.IsClass Then
                Target.Castclass(TargetType)
            Else
                Throw New NopeException
            End If
        End Sub

        ''' <summary>
        ''' Vergleicht die obersten beiden Werte auf dem Auswertungsstapel und beachtet die Art des Typs.
        ''' Legt True auf den Auswertungsstapel, wenn die Werte als gleich zu betrachten sind, andernfalls False.
        ''' Wenn TypeOfValues nicht angegeben wird, wird ein Aufruf an Object.Equals(Object, Object) verwendet.
        ''' </summary>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Sub CompareEquality(Target As ILGenerator)
            Target.CompareEquality(Nothing)
        End Sub

        ''' <summary>
        ''' Vergleicht die obersten beiden Werte auf dem Auswertungsstapel und beachtet die Art des Typs.
        ''' Legt True auf den Auswertungsstapel, wenn die Werte als gleich zu betrachten sind, andernfalls False.
        ''' Ist TypeOfValues Nothing, wird ein Aufruf an <see cref="Object"/>.Equals(Object, Object) verwendet.
        ''' Bei Wertetypen wird versucht, einen passenden Operator op_Equality (z.B. <see cref="System.Drawing.Color.op_Equality"/>) zu finden. Wird dieser nicht gefunden, wird <see cref="Object"/>.Equals(Object, Object) verwendet. Es ist zu beachten, dass dabei Boxing verwendet wird. Dazu wird eine lokale Variable des angegebenen Typs deklariert. Diese Variable wird wiederverwendet, wenn CompareEquality mit dem gleichen Typ erneut aufgerufen wird.
        ''' Bei Interfaces, Klassen (nicht String) und Object, wird der OpCode <see cref="System.Reflection.Emit.OpCodes.Ceq"/> verwendet.
        ''' Bei String wird <see cref="String"/>.Compare(String, String) verwendet und das Ergebnis mit auf Gleichheit mit 0 geprüft.
        ''' </summary>
        ''' <param name="TypeOfValues">Der Typ der zu vergleichenden Werte.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Sub CompareEquality(Target As ILGenerator, TypeOfValues As Type)
            If TypeOfValues Is Nothing Then
                Target.Call(PredefinedMembers.System.Object.Equals_Object_Object)
            ElseIf TypeOfValues.IsValueType Then
                Dim EqualityOperator = TypeOfValues.GetMethod("op_Equality", BindingFlags.Public Or BindingFlags.Static, Nothing, {TypeOfValues, TypeOfValues}, Nothing)
                If EqualityOperator Is Nothing Then
                    Dim Fields = CompareEqualityMethodCollection.GetFields(Target, TypeOfValues)
                    Target.Stloc(Fields.TargetLocal.Target)
                    Target.Box(TypeOfValues)
                    Target.Ldloc(Fields.TargetLocal.Target)
                    Target.Box(TypeOfValues)
                    Target.Call(PredefinedMembers.System.Object.Equals_Object_Object)
                Else
                    Target.Call(EqualityOperator)
                End If
            ElseIf TypeOfValues.IsInterface OrElse _
                   TypeOfValues Is GetType(Object) Then
                Target.Ceq()
            ElseIf TypeOfValues Is GetType(String) Then
                Target.Call(PredefinedMembers.System.String.Compare_String_String)
                Target.Ldc_I4_0()
                Target.Ceq()
            ElseIf TypeOfValues.IsClass Then
                Target.Ceq()
            Else
                Throw New NopeException
            End If
        End Sub

        ''' <summary>
        ''' Legt die Adresse des Objektes, das sich auf dem Auswertungsstapel befindet, in Form eines <see cref="IntPtr"/> auf den Auswertungsstapel.
        ''' Siehe <see cref="Helpers.GetObjectAddress"/>.
        ''' </summary>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Sub GetObjectAddress(Target As ILGenerator)
            With Target
                Dim Fields = GetObjectAddressMethodCollection.GetFields(Target)
                .Stloc(Fields.TargetLocal.Target)
                .Ldloca(Fields.TargetLocal.Target)
                .Mkrefany(GetType(Object))
                .Stloc(Fields.TypedReferenceLocal.Target)
                .Ldloca(Fields.TypedReferenceLocal.Target)
                .Conv_U()
                .Ldind_I()
                .Ldobj(GetType(IntPtr))
            End With
        End Sub

        ''' <summary>
        ''' Erstellt alle nötigen Member für ein Event. Es wird die standard-Namenskonvention von .Net verwendet.
        ''' </summary>
        ''' <param name="Target">Der Typ, in dem das Event und die dazugehörigen Member deklariert werden.</param>
        ''' <param name="AccessModifiers">Gibt den Sichtbarkeitsbereich des Events an.</param>
        ''' <param name="IsStatic">Gibt an, ob es sich um ein statisches Event handelt.</param>
        ''' <param name="EventName">Gibt den Namen des Events an.</param>
        ''' <param name="DelegateType">Gibt den Delegat-Typ des Events an. Muss von <see cref="System.EventHandler"/> erben.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function EmitEvent(Target As TypeBuilder, AccessModifiers As AccessModifiers, IsStatic As Boolean, EventName As String, DelegateType As Type) As EmitEvent
            If Not GetType(EventHandler).IsAssignableFrom(DelegateType) Then
                Throw New ArgumentException("Der in DelegateType angegebene Typ muss von System.EventHandler erben.", "DelegateType")
            End If
            Dim FA = FieldAttributes.Private
            Dim MA = Reflection.MethodAttributes.Final Or Reflection.MethodAttributes.SpecialName Or Reflection.MethodAttributes.NewSlot Or Reflection.MethodAttributes.Virtual Or Reflection.MethodAttributes.HideBySig
            Dim CC As CallingConventions
            Select Case AccessModifiers
                Case AccessModifiers.Private
                    MA = MA Or Reflection.MethodAttributes.Private
                Case AccessModifiers.Family
                    MA = MA Or Reflection.MethodAttributes.Family
                Case AccessModifiers.Assembly
                    MA = MA Or Reflection.MethodAttributes.Assembly
                Case AccessModifiers.FamilyAndAssembly
                    MA = MA Or Reflection.MethodAttributes.FamANDAssem
                Case AccessModifiers.FamilyOrAssembly
                    MA = MA Or Reflection.MethodAttributes.FamORAssem
                Case AccessModifiers.Public
                    MA = MA Or Reflection.MethodAttributes.Public
                Case Else
                    Throw New NopeException
            End Select
            If IsStatic Then
                FA = FA Or FieldAttributes.Static
                MA = MA Or MethodAttributes.Static
                CC = CallingConventions.Standard
            Else
                CC = CallingConventions.HasThis
            End If
            Dim EventField = Target.DefineField(EventName & "Event", DelegateType, FA)
            Dim AddHandlerMethod = Target.DefineMethod("add_" & EventName, MA, CC, GetType(Void), {DelegateType})
            With AddHandlerMethod.GetILGenerator
                If IsStatic Then
                    .Ldsfld(EventField)
                    .Ldarg_0()
                    .Call(PredefinedMembers.System.Delegate.Combine_Delegate_Delegate)
                    .Castclass(DelegateType)
                    .Stsfld(EventField)
                    .Ret()
                Else
                    .Ldarg_0()
                    .Ldarg_0()
                    .Ldfld(EventField)
                    .Ldarg_1()
                    .Call(PredefinedMembers.System.Delegate.Combine_Delegate_Delegate)
                    .Castclass(DelegateType)
                    .Stfld(EventField)
                    .Ret()
                End If
            End With
            Dim RemoveHandlerMethod = Target.DefineMethod("remove_" & EventName, MA, CC, GetType(Void), {DelegateType})
            With RemoveHandlerMethod.GetILGenerator
                If IsStatic Then
                    .Ldsfld(EventField)
                    .Ldarg_0()
                    .Call(PredefinedMembers.System.Delegate.Remove_Delegate_Delegate)
                    .Castclass(DelegateType)
                    .Stsfld(EventField)
                    .Ret()
                Else
                    .Ldarg_0()
                    .Ldarg_0()
                    .Ldfld(EventField)
                    .Ldarg_1()
                    .Call(PredefinedMembers.System.Delegate.Remove_Delegate_Delegate)
                    .Castclass(DelegateType)
                    .Stfld(EventField)
                    .Ret()
                End If
            End With
            Dim EventMember = Target.DefineEvent(EventName, EventAttributes.None, DelegateType)
            EventMember.SetAddOnMethod(AddHandlerMethod)
            EventMember.SetRemoveOnMethod(RemoveHandlerMethod)
            Return New EmitEvent(EventMember, EventField, AddHandlerMethod, RemoveHandlerMethod)
        End Function

    End Module

End Namespace
