﻿
Imports System.IO
Imports System.Runtime.CompilerServices

Namespace Extensions

    ''' <summary>
    ''' Beinhaltet hilfreiche Methoden.
    ''' </summary>
    Public Module Extensions

        ''' <summary>
        ''' Castet den Wert zu <paramref name="T"/>. Entspricht DirectCast(Target, T).
        ''' Hinweis: Unter Visual Basic funktioniert diese Extension nicht für Object.
        ''' Details siehe http://www.vb-paradise.de/sonstiges/off-topic/p838773-was-die-welt-wirklich-nicht-braucht-fun-links-usw/#post838773
        ''' </summary>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function AsT(Of T)(Target As Object) As T
            Return DirectCast(Target, T)
        End Function

        ''' <summary>
        ''' Schränkt dieses Objekt auf seine Member als <see cref="IEnumerable(Of T)"/> ein.
        ''' </summary>
        Public Function AsEnumerable(Of T)(Target As IEnumerable(Of T)) As IEnumerable(Of T)
            Return Target
        End Function

        ''' <summary>
        ''' Gibt die höherwertigen 8 Bits des 16-Bit-Wertes zurück.
        ''' Vorzeichenlos.
        ''' </summary>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function HByte(Target As UInt16) As Byte
            Return CByte(Target >> 8)
        End Function

        ''' <summary>
        ''' Gibt die niederwertigen 8 Bits des 16-Bit-Wertes zurück.
        ''' Vorzeichenlos.
        ''' </summary>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function LByte(Target As UInt16) As Byte
            Return CByte(Target And Byte.MaxValue)
        End Function

        ''' <summary>
        ''' Schreibt das angegebenen Byte in den Stream.
        ''' </summary>
        ''' <param name="Value">Das Byte, das in den Stream geschrieben wird.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Sub Write(Target As Stream, Value As Byte)
            Target.Write({Value}, 0, 1)
        End Sub

        ''' <summary>
        ''' Schreibt alle Bytes des angegebenen Arrays in den Stream.
        ''' </summary>
        ''' <param name="Bytes">Die Bytes, die in den Stream geschrieben werden.</param>
        ''' <exception cref="ArgumentException"><paramref name="Bytes"/> ist Nothing oder enthält keine Elemente.</exception>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Sub Write(Target As Stream, Bytes As Byte())
            If Bytes Is Nothing OrElse Bytes.Length = 0 Then
                Throw New ArgumentException("Das Array ist Nothing oder enthält keine Elemente", "Bytes")
            End If
            Target.Write(Bytes, 0, Bytes.Length)
        End Sub

        ''' <summary>
        ''' Liest ein einzelnes Byte aus dem Stream und es zurück.
        ''' </summary>
        ''' <exception cref="Streams.EndOfStreamException">Es konnte kein Byte aus dem Stream gelesen werden.</exception>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function Read(Target As Stream) As Byte
            Dim Temp = New Byte(0) {}
            Dim ReadBytes = Target.Read(Temp, 0, 1)
            If Not ReadBytes = 1 Then
                Throw New Streams.EndOfStreamException("Es konnte kein Byte aus dem Stream gelesen werden.")
            End If
            Return Temp(0)
        End Function

        ''' <summary>
        ''' Liest <paramref name="ByteCount"/> Bytes aus dem Stream und gibt ein Array zurück, das genau diese Bytes beinhaltet.
        ''' </summary>
        ''' <param name="ByteCount">Die Anzahl an Bytes, die aus dem Stream gelesen werden soll.</param>
        ''' <exception cref="ArgumentOutOfRangeException"><paramref name="ByteCount"/> ist 0 oder negativ.</exception>
        ''' <exception cref="Streams.EndOfStreamException">Es wurden weniger Bytes gelesen, als von <paramref name="ByteCount"/> angegeben.</exception>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function Read(Target As Stream, ByteCount As Integer) As Byte()
            If Not ByteCount > 0 Then
                Throw New ArgumentOutOfRangeException("ByteCount", ByteCount, "ByteCount darf nicht 0 oder negativ sein.")
            End If
            Dim Temp = New Byte(ByteCount - 1) {}
            Dim ReadBytes = Target.Read(Temp, 0, ByteCount)
            If Not ReadBytes = ByteCount Then
                Throw New Streams.EndOfStreamException("Die Anzahl an gelesenen Bytes (" & ReadBytes.ToString & ") stimmt nicht mit der angegebenen Anzahl (" & ByteCount.ToString & ") überein.")
            End If
            Return Temp
        End Function

        ''' <summary>
        ''' Kopiert alle Elemente aus <paramref name="Other"/> in dieses Array, beginnend bei Index <paramref name="TargetIndex"/>.
        ''' </summary>
        ''' <param name="Other">Das Array aus dem die Elemente kopiert werden.</param>
        ''' <param name="TargetIndex">Der Index, an dem begonnen wird, die Elemente in das Ziel-Array zu schreiben.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Sub Insert(Of T)(Target As T(), Other As T(), TargetIndex As Integer)
            Array.Copy(Other, 0, Target, TargetIndex, Other.Length)
        End Sub

        ''' <summary>
        ''' Gibt ein Array zurück, das <paramref name="Length"/> Elemente dieses Arrays, beginnend bei <paramref name="StartIndex"/>, enthält.
        ''' </summary>
        ''' <param name="StartIndex">Der Index des ersten Elementes im Quell-Array.</param>
        ''' <param name="Length">Die Anzahl der zurückgegebenen Elemente.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function SubSequence(Of T)(Target As T(), StartIndex As Integer, Length As Integer) As T()
            Dim Temp = New T(Length - 1) {}
            Array.Copy(Target, StartIndex, Temp, 0, Length)
            Return Temp
        End Function

        ''' <summary>
        ''' Gibt eine Auflistung zurück, die <paramref name="Length"/> Elemente dieser Auflistung, beginnend bei <paramref name="StartIndex"/>, enthält.
        ''' </summary>
        ''' <param name="StartIndex">Der Index des ersten Elementes in der Quell-Auflistung.</param>
        ''' <param name="Length">Die Anzahl der zurückgegebenen Elemente.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function SubSequence(Of T)(Target As IEnumerable(Of T), StartIndex As Integer, Length As Integer) As IEnumerable(Of T)
            If StartIndex = 0 Then
                Return Target.Take(Length)
            Else
                Return Target.Skip(StartIndex).Take(Length)
            End If
        End Function

        ''' <summary>
        ''' Konvertiert den String in einen Enumwert des angegebenen Types <paramref name="T"/>.
        ''' </summary>
        ''' <typeparam name="T">Der Enum-Typ, in den der String konvertiert wird.</typeparam>
        ''' <exception cref="ArgumentException">T ist kein Enum-Typ.</exception>
        ''' <exception cref="ArgumentException">Es gibt keinen passenden Wert im Enum.</exception>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function ParseToEnum(Of T As Structure)(Target As String) As T
            If Not GetType(T).IsEnum Then
                Throw New ArgumentException("T muss ein Enum-Typ sein", "T")
            End If
            Dim Temp As T
            If [Enum].TryParse(Target, Temp) Then
                Return Temp
            End If
            Throw New ArgumentException("'" & Target & "' kann nicht zu " & GetType(T).Name & " konvertiert werden.")
        End Function

        ''' <summary>
        ''' Versucht, den String in einen Enumwert des angegebenen Types <paramref name="T"/> zu konvertieren.
        ''' Gibt bei Erfolg True zurück, andernfalls False.
        ''' </summary>
        ''' <typeparam name="T">Der Enum-Typ, in den der String konvertiert wird.</typeparam>
        ''' <param name="OutputValue">Eine Referenz auf ein Feld des Types <paramref name="T"/>, in dem das Ergebnis gespeichert wird.</param>
        ''' <exception cref="ArgumentException">T ist kein Enum-Typ.</exception>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function TryParseToEnum(Of T As Structure)(Target As String, OutputValue As T) As Boolean
            If Not GetType(T).IsEnum Then
                Throw New ArgumentException("T muss ein Enum-Typ sein", "T")
            End If
            Return [Enum].TryParse(Of T)(Target, OutputValue)
        End Function

        ''' <summary>
        ''' Gibt, wenn vorhanden, den Index nach dem ersten Vorkommen von <paramref name="ContainedString"/> zurück, ansonsten -1.
        ''' </summary>
        ''' <param name="ContainedString">Der String, nach dem gesucht wird.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function IndexAfter(Target As String, ContainedString As String) As Integer
            Dim Index = Target.IndexOf(ContainedString)
            If Index = -1 Then
                Return -1
            End If
            Return Index + ContainedString.Length
        End Function

        ''' <summary>
        ''' Gibt den verbleibenden String nach dem ersten Vorkommen von <paramref name="ContainedString"/> bis zum Ende zurück.
        ''' Ist <paramref name="ContainedString"/> nicht im Quellstring vorhanden oder befindet er sich an dessen Ende, wird ein leerer String zurückgegeben.
        ''' </summary>
        ''' <param name="ContainedString">Der String, nach dem gesucht wird.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function SubstringAfter(Target As String, ContainedString As String) As String
            Dim EndIndex = Target.IndexAfter(ContainedString)
            If EndIndex = -1 Then
                Return String.Empty
            End If
            If EndIndex >= Target.Length Then
                Return String.Empty
            End If
            Return Target.Substring(EndIndex)
        End Function

        ''' <summary>
        ''' Gibt den String nach dem ersten Vorkommen von <paramref name="ContainedString"/> mit der angegebenen Länge zurück.
        ''' Ist <paramref name="ContainedString"/> nicht im Quellstring vorhanden oder befindet er sich an dessen Ende, wird ein leerer String zurückgegeben.
        ''' Reicht <paramref name="Length"/> über das Ende des Strings hinaus, wird eine Exception ausgelöst. Siehe <see cref="String.SubString"/>.
        ''' </summary>
        ''' <param name="ContainedString">Der String, nach dem gesucht wird.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function SubstringAfter(Target As String, ContainedString As String, Length As Integer) As String
            Dim EndIndex = Target.IndexAfter(ContainedString)
            If EndIndex = -1 Then
                Return String.Empty
            End If
            If EndIndex >= Target.Length Then
                Return String.Empty
            End If
            Return Target.Substring(EndIndex, Length)
        End Function

        ''' <summary>
        ''' Gibt den enthaltenen String, beginnend bei <paramref name="StartIndex"/> und endend bei <paramref name="EndIndex"/>, zurück.
        ''' Liegt <paramref name="EndIndex"/> außerhalb des Strings oder vor <paramref name="StartIndex"/>, wird eine <see cref="ArgumentOutOfRangeException"/> ausgelöst.
        ''' </summary>
        ''' <param name="StartIndex">Der Index des ersten Zeichens des zurückgegebenen Strings.</param>
        ''' <param name="EndIndex">Der Index des letzten Zeichens des zurückgegebenen Strings.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function SubstringBetween(Target As String, StartIndex As Integer, EndIndex As Integer) As String
            If EndIndex >= Target.Length Then
                Throw New ArgumentOutOfRangeException("EndIndex", EndIndex, "EndIndex muss innerhalb des Stings liegen (maximal " & (Target.Length - 1).ToString & ")")
            End If
            If EndIndex < StartIndex Then
                Throw New ArgumentOutOfRangeException("EndIndex", EndIndex, "EndIndex muss größer oder gleich StartIndex (" & StartIndex.ToString & ") sein")
            End If
            Return Target.Substring(StartIndex, EndIndex - StartIndex + 1)
        End Function

        ''' <summary>
        ''' Gibt eine Liste zurück, dass alle Elemente aller in <paramref name="Lists"/> angegebenen Listen enthält.
        ''' </summary>
        ''' <param name="Target">Die einzelnen Listen, die miteinander verbunden werden sollen.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function ConcatAll(Of T)(Target As IEnumerable(Of IEnumerable(Of T))) As IEnumerable(Of T)
            Dim Enumerator As IEnumerator(Of IEnumerable(Of T)) = Nothing
            Try
                Enumerator = Target.GetEnumerator
                If Not Enumerator.MoveNext Then
                    Return New T() {}
                End If
                Dim Current = Enumerator.Current
                Do While Enumerator.MoveNext
                    Current = Enumerable.Concat(Current, Enumerator.Current)
                Loop
                Return Current
            Finally
                If Not Enumerator Is Nothing Then
                    Enumerator.Dispose()
                End If
            End Try
        End Function

        ''' <summary>
        ''' Gibt eine Liste zurück, die mit <paramref name="Item"/> beginnt und anschließend die bestehenden Elemente enthält.
        ''' </summary>
        ''' <param name="Item">Das Element, das am Beginn eingefügt wird.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function Prepend(Of T)(Target As IEnumerable(Of T), Item As T) As IEnumerable(Of T)
            Return Enumerable.Concat({Item}, Target)
        End Function

        ''' <summary>
        ''' Gibt eine Liste zurück, die mit den bestehenden Elementen beginnt und anschließend <paramref name="Item"/> enthält.
        ''' </summary>
        ''' <param name="Item">Das Element, das am Ende angehängt wird.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function Append(Of T)(Target As IEnumerable(Of T), Item As T) As IEnumerable(Of T)
            Return Enumerable.Concat(Target, {Item})
        End Function

        ''' <summary>
        ''' Gibt eine Liste von Indices, an denen die <paramref name="SubSequence"/>-Sequenzen in der <paramref name="Target"/>-Sequenz beginnen, zurück.
        ''' </summary>
        ''' <param name="SubSequence">Die Unter-Sequenz, deren Indices in der Sequenz gesucht werden.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function AllIndicesOf(Of T)(Target As IEnumerable(Of T), SubSequence As IEnumerable(Of T)) As IEnumerable(Of Integer)
            Return AllIndicesOf(Target, SubSequence, 0)
        End Function

        ''' <summary>
        ''' Gibt eine Liste von Indices, an denen die <paramref name="SubSequence"/>-Sequenzen in der <paramref name="Target"/>-Sequenz beginnen, zurück, beginnend bei <paramref name="StartIndex"/>.
        ''' </summary>
        ''' <param name="SubSequence">Die Unter-Sequenz, deren Indices in der Sequenz gesucht werden.</param>
        ''' <param name="StartIndex">Der Index, von dem an gesucht wird. Dieser Index kann in der zurückgegebenen Liste enthalten sein.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function AllIndicesOf(Of T)(Target As IEnumerable(Of T), SubSequence As IEnumerable(Of T), StartIndex As Integer) As IEnumerable(Of Integer)
            Return AllIndicesOf(Target.ToArray, SubSequence.ToArray, StartIndex)
        End Function

        ''' <summary>
        ''' Gibt eine Liste von Indices, an denen die <paramref name="SubSequence"/>-Sequenzen in der <paramref name="Target"/>-Sequenz beginnen, zurück.
        ''' </summary>
        ''' <param name="SubSequence">Die Unter-Sequenz, deren Indices in der Sequenz gesucht werden.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function AllIndicesOf(Of T)(Target As T(), SubSequence As T()) As IEnumerable(Of Integer)
            Return AllIndicesOf(Target, SubSequence, 0)
        End Function

        ''' <summary>
        ''' Gibt eine Liste von Indices, an denen die <paramref name="SubSequence"/>-Sequenzen in der <paramref name="Target"/>-Sequenz beginnen, zurück, beginnend bei <paramref name="StartIndex"/>.
        ''' </summary>
        ''' <param name="SubSequence">Die Unter-Sequenz, deren Indices in der Sequenz gesucht werden.</param>
        ''' <param name="StartIndex">Der Index, von dem an gesucht wird. Dieser Index kann in der zurückgegebenen Liste enthalten sein.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function AllIndicesOf(Of T)(Target As T(), SubSequence As T(), StartIndex As Integer) As IEnumerable(Of Integer)
            Dim Indices As New List(Of Integer)
            If SubSequence.Length <= Target.Length Then
                For i = 0 To Target.Length - SubSequence.Length
                    Dim IsValid = True
                    For j = 0 To SubSequence.Length - 1
                        If Not Object.Equals(Target(i + j), SubSequence(j)) Then
                            IsValid = False
                            Exit For
                        End If
                    Next
                    If IsValid Then
                        Indices.Add(i)
                    End If
                Next
            End If
            Return Indices
        End Function

        ''' <summary>
        ''' Verknüpft alle Auflistungen in dieser Auflistung mit <paramref name="Concatenator"/> dazwischen.
        ''' Enthält diese Auflistung 0 Elemente, wird ein leeres Array von T zurückgegeben.
        ''' Enthält diese Auflistung 1 Element, wird nur dieses eine Element zurückgegeben.
        ''' Beispiel: {{1, 2}, {3, 4}, {5, 6}}.Join({7, 8}) -> {1, 2, 7, 8, 3, 4, 7, 8, 5, 6}
        ''' </summary>
        ''' <param name="Concatenator">Die Elemente zwischen den Elementen dieser Auflistung.</param>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function Join(Of T)(Target As IEnumerable(Of IEnumerable(Of T)), Concatenator As IEnumerable(Of T)) As IEnumerable(Of T)
            If Target Is Nothing Then
                Throw New ArgumentNullException("Target", "Die Auflistung darf nicht Nothing sein.")
            End If
            If Concatenator Is Nothing Then
                Throw New ArgumentNullException("Concatenator", "Concatenator darf nicht Nothing sein.")
            End If
            Select Case Target.Count
                Case 0
                    Return New T() {}
                Case 1
                    Return Target(0)
                Case Else
                    Dim Temp As IEnumerable(Of T) = Nothing
                    For Each i In Target
                        If Temp Is Nothing Then
                            Temp = i
                        Else
                            Temp = Enumerable.Concat(Temp, Enumerable.Concat(Concatenator, i))
                        End If
                    Next
                    Return Temp
            End Select
        End Function

        ''' <summary>
        ''' Gibt <see cref="System.Windows.Visibility.Visible"/> zurück, wenn dieser Boolean True ist, andernfalls <see cref="System.Windows.Visibility.Collapsed"/>.
        ''' </summary>
        <DebuggerStepThrough()> _
        <Extension()> _
        Public Function ToVisibility(Target As Boolean) As System.Windows.Visibility
            If Target Then
                Return Windows.Visibility.Visible
            Else
                Return Windows.Visibility.Collapsed
            End If
        End Function

    End Module

End Namespace
