﻿
''' <summary>
''' Beinhaltet Methoden zum proportionalen Skalieren von Rechtecken.
''' </summary>
Public NotInheritable Class Resizing
    Private Sub New()
    End Sub

    ''' <summary>
    ''' Eine Angabe über die Art, wie <see cref="Resize"/> skaliert.
    ''' </summary>
    Public Enum ResizeMode As Integer
        ''' <summary>
        ''' Das ursprüngliche Rechteck wird so skaliert, dass es vollständig in das Zielrechteck passt.
        ''' Breite und Höhe des ursprünglichen Rechteckes sind nie größer als Breite und Höhe des Zielrechtecks.
        ''' </summary>
        Uniform = 1
        ''' <summary>
        ''' Das ursprüngliche Rechteck wird so skaliert, dass es das Zielrechteck vollständig füllt.
        ''' Breite und Höhe des ursprünglichen Rechteckes sind nie kleiner als Breite und Höhe des Zielrechtecks.
        ''' </summary>
        UniformToFill = 2
    End Enum

    ''' <summary>
    ''' Gibt ein Rechteck mit dem Seitenverhältnis <paramref name="SourceHeight"/> zu <paramref name="SourceWidth"/> zurück, welches sich in der Mitte des durch <paramref name="TargetX"/>, <paramref name="TargetY"/>, <paramref name="TargetWidth"/> und <paramref name="TargetHeight"/> angegebenen Rechtecks befindet.
    ''' Die Größe des neuen Rechtecks wird durch <paramref name="Mode"/> bestimmt. Siehe <see cref="ResizeMode"/>
    ''' </summary>
    ''' <param name="SourceWidth">Die Breite des ursprünglichen Rechtecks.
    ''' Oder: Der X-Anteil des Seitenverhältnises.</param>
    ''' <param name="SourceHeight">Die Höhe des ursprünglichen Rechtecks.
    ''' Oder: Der Y-Anteil des Seitenverhältnises.</param>
    ''' <param name="TargetX">Die X-Koordinate der linken oberen Ecke des Zielrechtecks.</param>
    ''' <param name="TargetY">Die Y-Koordinate der linken oberen Ecke des Zielrechtecks.</param>
    ''' <param name="TargetWidth">Die Breite des Zielrechtecks.</param>
    ''' <param name="TargetHeight">Die Höhe des Zielrechtecks.</param>
    ''' <param name="Mode">Gibt an, wie skaliert wird. Siehe <see cref="ResizeMode"/>.</param>
    <DebuggerStepThrough()> _
    Public Shared Function Resize(SourceWidth As Integer, SourceHeight As Integer, TargetX As Integer, TargetY As Integer, TargetWidth As Integer, TargetHeight As Integer, Optional Mode As ResizeMode = ResizeMode.Uniform) As System.Drawing.Rectangle
        Dim NewWidth As Integer
        Dim NewHeight As Integer
        Dim NewX As Integer
        Dim NewY As Integer
        Dim TargetRatio = TargetHeight / TargetWidth
        Dim OriginalRatio = SourceHeight / SourceWidth
        If TargetRatio < OriginalRatio Then
            Select Case Mode
                Case ResizeMode.Uniform
                    NewWidth = TargetHeight * SourceWidth \ SourceHeight
                    NewHeight = TargetHeight
                    NewX = (TargetWidth - NewWidth) \ 2
                    NewY = 0
                Case ResizeMode.UniformToFill
                    NewWidth = TargetWidth
                    NewHeight = TargetWidth * SourceHeight \ SourceWidth
                    NewX = 0
                    NewY = (TargetHeight - NewHeight) \ 2
                Case Else
                    Throw New NopeException
            End Select
        ElseIf TargetRatio > OriginalRatio Then
            Select Case Mode
                Case ResizeMode.Uniform
                    NewWidth = TargetWidth
                    NewHeight = TargetWidth * SourceHeight \ SourceWidth
                    NewX = 0
                    NewY = (TargetHeight - NewHeight) \ 2
                Case ResizeMode.UniformToFill
                    NewWidth = TargetHeight * SourceWidth \ SourceHeight
                    NewHeight = TargetHeight
                    NewX = (TargetWidth - NewWidth) \ 2
                    NewY = 0
                Case Else
                    Throw New NopeException
            End Select
        Else
            NewWidth = TargetWidth
            NewHeight = TargetHeight
            NewX = 0
            NewY = 0
        End If
        Return New System.Drawing.Rectangle(TargetX + NewX, TargetY + NewY, NewWidth, NewHeight)
    End Function

    ''' <summary>
    ''' Gibt ein Rechteck mit dem Seitenverhältnis <paramref name="SourceHeight"/> zu <paramref name="SourceWidth"/> zurück, welches sich in der Mitte des durch <paramref name="TargetX"/>, <paramref name="TargetY"/>, <paramref name="TargetWidth"/> und <paramref name="TargetHeight"/> angegebenen Rechtecks befindet.
    ''' Die Größe des neuen Rechtecks wird durch <paramref name="Mode"/> bestimmt. Siehe <see cref="ResizeMode"/>
    ''' </summary>
    ''' <param name="SourceWidth">Die Breite des ursprünglichen Rechtecks.
    ''' Oder: Der X-Anteil des Seitenverhältnises.</param>
    ''' <param name="SourceHeight">Die Höhe des ursprünglichen Rechtecks.
    ''' Oder: Der Y-Anteil des Seitenverhältnises.</param>
    ''' <param name="TargetX">Die X-Koordinate der linken oberen Ecke des Zielrechtecks.</param>
    ''' <param name="TargetY">Die Y-Koordinate der linken oberen Ecke des Zielrechtecks.</param>
    ''' <param name="TargetWidth">Die Breite des Zielrechtecks.</param>
    ''' <param name="TargetHeight">Die Höhe des Zielrechtecks.</param>
    ''' <param name="Mode">Gibt an, wie skaliert wird. Siehe <see cref="ResizeMode"/>.</param>
    <DebuggerStepThrough()> _
    Public Shared Function Resize(SourceWidth As Single, SourceHeight As Single, TargetX As Single, TargetY As Single, TargetWidth As Single, TargetHeight As Single, Optional Mode As ResizeMode = ResizeMode.Uniform) As System.Drawing.RectangleF
        Dim NewWidth As Single
        Dim NewHeight As Single
        Dim NewX As Single
        Dim NewY As Single
        Dim TargetRatio = TargetHeight / TargetWidth
        Dim OriginalRatio = SourceHeight / SourceWidth
        If TargetRatio < OriginalRatio Then
            Select Case Mode
                Case ResizeMode.Uniform
                    NewWidth = TargetHeight * SourceWidth / SourceHeight
                    NewHeight = TargetHeight
                    NewX = (TargetWidth - NewWidth) / 2
                    NewY = 0
                Case ResizeMode.UniformToFill
                    NewWidth = TargetWidth
                    NewHeight = TargetWidth * SourceHeight / SourceWidth
                    NewX = 0
                    NewY = (TargetHeight - NewHeight) / 2
                Case Else
                    Throw New NopeException
            End Select
        ElseIf TargetRatio > OriginalRatio Then
            Select Case Mode
                Case ResizeMode.Uniform
                    NewWidth = TargetWidth
                    NewHeight = TargetWidth * SourceHeight / SourceWidth
                    NewX = 0
                    NewY = (TargetHeight - NewHeight) / 2
                Case ResizeMode.UniformToFill
                    NewWidth = TargetHeight * SourceWidth / SourceHeight
                    NewHeight = TargetHeight
                    NewX = (TargetWidth - NewWidth) / 2
                    NewY = 0
                Case Else
                    Throw New NopeException
            End Select
        Else
            NewWidth = TargetWidth
            NewHeight = TargetHeight
            NewX = 0
            NewY = 0
        End If
        Return New System.Drawing.RectangleF(TargetX + NewX, TargetY + NewY, NewWidth, NewHeight)
    End Function

    ''' <summary>
    ''' Gibt ein Rechteck mit dem Seitenverhältnis <paramref name="SourceHeight"/> zu <paramref name="SourceWidth"/> zurück, welches sich in der Mitte des durch <paramref name="TargetX"/>, <paramref name="TargetY"/>, <paramref name="TargetWidth"/> und <paramref name="TargetHeight"/> angegebenen Rechtecks befindet.
    ''' Die Größe des neuen Rechtecks wird durch <paramref name="Mode"/> bestimmt. Siehe <see cref="ResizeMode"/>
    ''' </summary>
    ''' <param name="SourceWidth">Die Breite des ursprünglichen Rechtecks.
    ''' Oder: Der X-Anteil des Seitenverhältnises.</param>
    ''' <param name="SourceHeight">Die Höhe des ursprünglichen Rechtecks.
    ''' Oder: Der Y-Anteil des Seitenverhältnises.</param>
    ''' <param name="TargetX">Die X-Koordinate der linken oberen Ecke des Zielrechtecks.</param>
    ''' <param name="TargetY">Die Y-Koordinate der linken oberen Ecke des Zielrechtecks.</param>
    ''' <param name="TargetWidth">Die Breite des Zielrechtecks.</param>
    ''' <param name="TargetHeight">Die Höhe des Zielrechtecks.</param>
    ''' <param name="Mode">Gibt an, wie skaliert wird. Siehe <see cref="ResizeMode"/>.</param>
    <DebuggerStepThrough()> _
    Public Shared Function Resize(SourceWidth As Double, SourceHeight As Double, TargetX As Double, TargetY As Double, TargetWidth As Double, TargetHeight As Double, Optional Mode As ResizeMode = ResizeMode.Uniform) As RectangleD
        Dim NewWidth As Double
        Dim NewHeight As Double
        Dim NewX As Double
        Dim NewY As Double
        Dim TargetRatio = TargetHeight / TargetWidth
        Dim OriginalRatio = SourceHeight / SourceWidth
        If TargetRatio < OriginalRatio Then
            Select Case Mode
                Case ResizeMode.Uniform
                    NewWidth = TargetHeight * SourceWidth / SourceHeight
                    NewHeight = TargetHeight
                    NewX = (TargetWidth - NewWidth) / 2
                    NewY = 0
                Case ResizeMode.UniformToFill
                    NewWidth = TargetWidth
                    NewHeight = TargetWidth * SourceHeight / SourceWidth
                    NewX = 0
                    NewY = (TargetHeight - NewHeight) / 2
                Case Else
                    Throw New NopeException
            End Select
        ElseIf TargetRatio > OriginalRatio Then
            Select Case Mode
                Case ResizeMode.Uniform
                    NewWidth = TargetWidth
                    NewHeight = TargetWidth * SourceHeight / SourceWidth
                    NewX = 0
                    NewY = (TargetHeight - NewHeight) / 2
                Case ResizeMode.UniformToFill
                    NewWidth = TargetHeight * SourceWidth / SourceHeight
                    NewHeight = TargetHeight
                    NewX = (TargetWidth - NewWidth) / 2
                    NewY = 0
                Case Else
                    Throw New NopeException
            End Select
        Else
            NewWidth = TargetWidth
            NewHeight = TargetHeight
            NewX = 0
            NewY = 0
        End If
        Return New RectangleD(TargetX + NewX, TargetY + NewY, NewWidth, NewHeight)
    End Function

    ''' <summary>
    ''' Speichert einen Satz von vier Gleitkommazahlen, die die Position und Größe eines Rechtecks angeben.
    ''' </summary>
    Public Structure RectangleD

        Private _X As Double
        ''' <summary>
        ''' Die X-Koordinate der linken oberen Ecke des Rechtecks.
        ''' </summary>
        Public ReadOnly Property X As Double
            <DebuggerStepThrough()> _
            Get
                Return _X
            End Get
        End Property

        Private _Y As Double
        ''' <summary>
        ''' Die Y-Koordinate der linken oberen Ecke des Rechtecks.
        ''' </summary>
        Public ReadOnly Property Y As Double
            <DebuggerStepThrough()> _
            Get
                Return _Y
            End Get
        End Property

        Private _Width As Double
        ''' <summary>
        ''' Die Breite des Rechtecks.
        ''' </summary>
        Public ReadOnly Property Width As Double
            <DebuggerStepThrough()> _
            Get
                Return _Width
            End Get
        End Property

        Private _Height As Double
        ''' <summary>
        ''' Die Höhe des Rechtecks.
        ''' </summary>
        Public ReadOnly Property Height As Double
            <DebuggerStepThrough()> _
            Get
                Return _Height
            End Get
        End Property

        ''' <summary>
        ''' Die X-Position des linken Randes des Rechtecks.
        ''' </summary>
        Public ReadOnly Property Left As Double
            <DebuggerStepThrough()> _
            Get
                Return _X
            End Get
        End Property

        ''' <summary>
        ''' Die Y-Position des oberen Randes des Rechtecks.
        ''' </summary>
        Public ReadOnly Property Top As Double
            <DebuggerStepThrough()> _
            Get
                Return _Y
            End Get
        End Property

        ''' <summary>
        ''' Die X-Position des rechten Randes des Rechtecks.
        ''' </summary>
        Public ReadOnly Property Right As Double
            <DebuggerStepThrough()> _
            Get
                Return _X + _Width
            End Get
        End Property

        ''' <summary>
        ''' Die Y-Position des unteren Randes des Rechtecks.
        ''' </summary>
        Public ReadOnly Property Bottom As Double
            <DebuggerStepThrough()> _
            Get
                Return _Y + _Height
            End Get
        End Property

        ''' <summary>
        ''' Konstruktor.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Sub New(NewX As Double, NewY As Double, NewWidth As Double, NewHeight As Double)
            _X = NewX
            _Y = NewY
            _Width = NewWidth
            _Height = NewHeight
        End Sub

        ''' <summary>
        ''' Prüft, ob die angegebenen Koordinaten innerhalb dieses Rechtecks liegen.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Function Contains(X As Double, Y As Double) As Boolean
            Return _X <= X AndAlso X < _X + _Width AndAlso _Y <= Y AndAlso Y < _Y + _Height
        End Function

        ''' <summary>
        ''' Prüft, ob das angegebene Rechteck vollständig innerhalb dieses Rechtecks liegt.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Function Contains(Other As RectangleD) As Boolean
            Return _X <= Other.X AndAlso Other.X + Other.Width <= _X + _Width AndAlso _Y <= Other.Y AndAlso Other.Y + Other.Height <= _Y + _Height
        End Function

        ''' <summary>
        ''' Konvertiert die Koordinaten und Größen zu Single und gibt ein <see cref="System.Drawing.RectangleF"/> zurück.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Function ToRectangleF() As System.Drawing.RectangleF
            Return New System.Drawing.RectangleF(CSng(_X), CSng(_Y), CSng(_Width), CSng(_Height))
        End Function

        ''' <summary>
        ''' Gibt den von den beiden angegebenen Rechtecken eingeschlossenen Bereich zurück.
        ''' Wenn sich die Rechtecke nicht schneiden, wird ein leeres <see cref="RectangleD"/> zurückgegeben.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Shared Function Intersect(A As RectangleD, B As RectangleD) As RectangleD
            Dim num1 = Math.Max(A._X, B._X)
            Dim num2 = Math.Min(A._X + A._Width, B._X + B._Width)
            Dim num3 = Math.Max(A._Y, B._Y)
            Dim num4 = Math.Min(A._Y + A._Height, B._Y + B._Height)
            If num2 >= num1 AndAlso num4 >= num3 Then
                Return New RectangleD(num1, num3, num2 - num1, num4 - num3)
            End If
            Return New RectangleD
        End Function

        ''' <summary>
        ''' Gibt ein Rechteck zurück, das die beiden angegebenen Rechtecke umschließt.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Shared Function Union(A As RectangleD, B As RectangleD) As RectangleD
            Dim num1 = Math.Min(A._X, B._X)
            Dim num2 = Math.Max(A._X + A._Width, B._X + B._Width)
            Dim num3 = Math.Min(A._Y, B._Y)
            Dim num4 = Math.Max(A._Y + A._Height, B._Y + B._Height)
            Return New RectangleD(num1, num3, num2 - num1, num4 - num3)
        End Function

        ''' <summary>
        ''' Prüft, ob die beiden Rechtecke die selbe Position und Größe besitzen.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Shared Operator =(A As RectangleD, B As RectangleD) As Boolean
            Return A._X = B._X AndAlso A._Y = B._Y AndAlso A._Width = B._Width AndAlso A._Height = B._Height
        End Operator

        ''' <summary>
        ''' Prüft, ob die beiden Rechtecke eine unterschiedliche Position oder Größe besitzen.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Shared Operator <>(A As RectangleD, B As RectangleD) As Boolean
            Return A._X <> B._X OrElse A._Y <> B._Y OrElse A._Width <> B._Width OrElse A._Height <> B._Height
        End Operator

    End Structure

End Class
