﻿
Imports OnUtils.Extensions

Namespace Streams

    ''' <summary>
    ''' Bietet Methoden zum Lesen von Strings in Blöcken.
    ''' </summary>
    Public Class StringStream

        Dim IsInitialized As Boolean = False

        Dim _SourceString As String
        ''' <summary>
        ''' Gibt den Quellstring an.
        ''' </summary>
        Public ReadOnly Property SourceString As String
            <DebuggerStepThrough()> _
            Get
                EnsureIsInitialized()
                Return _SourceString
            End Get
        End Property

        Dim _CurrentIndex As Integer
        ''' <summary>
        ''' Gibt den Index des nächsten Zeichens im String an.
        ''' </summary>
        Public ReadOnly Property CurrentIndex As Integer
            <DebuggerStepThrough()> _
            Get
                EnsureIsInitialized()
                Return _CurrentIndex
            End Get
        End Property

        ''' <summary>
        ''' Gibt das nächste Zeichen im Stream an.
        ''' </summary>
        Public ReadOnly Property Current As Char
            <DebuggerStepThrough()> _
            Get
                EnsureIsInitialized()
                EnsureNotEndOfStream()
                Return _SourceString(_CurrentIndex)
            End Get
        End Property

        ''' <summary>
        ''' Gibt an, ob das Ende des Streams erreicht wurde.
        ''' </summary>
        Public ReadOnly Property EndOfStream As Boolean
            <DebuggerStepThrough()> _
            Get
                Return _CurrentIndex >= _SourceString.Length
            End Get
        End Property

        ''' <summary>
        ''' Gibt die Anzahl an verbleibenden Zeichen im Stream zurück.
        ''' </summary>
        Public ReadOnly Property RemainingChars As Integer
            <DebuggerStepThrough()> _
            Get
                EnsureIsInitialized()
                Return _SourceString.Length - _CurrentIndex
            End Get
        End Property

        ''' <summary>
        ''' Konstruktor für abgeleitete Klassen. Rufen Sie <see cref="Initialize"/> auf, bevor Sie den Stream verwenden.
        ''' </summary>
        Protected Sub New()
        End Sub

        ''' <summary>
        ''' Konstruktor.
        ''' </summary>
        ''' <param name="NewSourceString">Der Quellstring, der verarbeitet wird.</param>
        <DebuggerStepThrough()> _
        Public Sub New(NewSourceString As String)
            Initialize(NewSourceString)
        End Sub

        ''' <summary>
        ''' Initialisiert den StringStream mit dem angegebenen String. Darf von abgeleiteten Klassen nur aufgerufen werden, wenn der parameterlose Konstruktor verwendet wurde.
        ''' </summary>
        ''' <param name="NewSourceString">Der Quellstring, der verarbeitet wird.</param>
        <DebuggerStepThrough()> _
        Protected Sub Initialize(NewSourceString As String)
            If IsInitialized Then
                Throw New InvalidOperationException("Der StringStream ist bereits initialisiert.")
            End If
            If NewSourceString Is Nothing Then
                Throw New ArgumentNullException("NewSourceString")
            End If
            _SourceString = NewSourceString
            _CurrentIndex = 0
            IsInitialized = True
        End Sub

        <DebuggerStepThrough()> _
        Private Sub EnsureNotEndOfStream()
            If EndOfStream Then
                Throw New EndOfStreamException
            End If
        End Sub

        <DebuggerStepThrough()> _
        Private Sub EnsureIsInitialized()
            If Not IsInitialized Then
                Throw New InvalidOperationException("Der Stream ist noch nicht initialisiert. Rufen Sie vor der Verwendung Initialize(String) auf.")
            End If
        End Sub

        ''' <summary>
        ''' Überspringt das nächste Zeichen im Stream.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Sub Advance()
            EnsureIsInitialized()
            If _CurrentIndex >= _SourceString.Length Then
                Throw New EndOfStreamException
            End If
            _CurrentIndex += 1
        End Sub

        ''' <summary>
        ''' Überspringt die nächsten <paramref name="Count"/> Zeichen im Stream.
        ''' </summary>
        ''' <param name="Count">Die Anzahl an Zeichen, die übersprungen werden.</param>
        <DebuggerStepThrough()> _
        Public Sub Advance(Count As Integer)
            EnsureIsInitialized()
            If _CurrentIndex + Count > _SourceString.Length Then
                Throw New EndOfStreamException
            End If
            _CurrentIndex += Count
        End Sub

        ''' <summary>
        ''' Gibt das nächste Zeichen im String zurück, ohne es zu entfernen.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Function ReadChar() As Char
            EnsureIsInitialized()
            EnsureNotEndOfStream()
            Dim Temp = Current
            _CurrentIndex += 1
            Return Temp
        End Function

        ''' <summary>
        ''' Gibt die nächsten <paramref name="Count"/> Zeichen zurück, ohne sie zu entfernen.
        ''' Wird das Ende des Streams erreicht, bevor genügend Zeichen gelesen werden, wird eine <see cref="EndOfStreamException"/> ausgelöst.
        ''' </summary>
        ''' <param name="Count">Die Anzahl an zu lesenden Zeichen.</param>
        <DebuggerStepThrough()> _
        Public Function Peek(Count As Integer) As String
            EnsureIsInitialized()
            If _CurrentIndex + Count > _SourceString.Length Then
                Throw New EndOfStreamException
            End If
            Return _SourceString.Substring(_CurrentIndex, Count)
        End Function

        ''' <summary>
        ''' Gibt die nächsten <paramref name="Count"/> Zeichen zurück.
        ''' Wird das Ende des Streams erreicht, bevor genügend Zeichen gelesen werden, wird eine <see cref="EndOfStreamException"/> ausgelöst.
        ''' </summary>
        ''' <param name="Count">Die Anzahl an zu lesenden Zeichen.</param>
        <DebuggerStepThrough()> _
        Public Function Read(Count As Integer) As String
            Dim Temp = Peek(Count)
            _CurrentIndex += Count
            Return Temp
        End Function

        ''' <summary>
        ''' Gibt die verbleibenden Zeichen bis zum Ende zurück, ohne sie zu entfernen.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Function PeekToEnd() As String
            EnsureIsInitialized()
            EnsureNotEndOfStream()
            Return _SourceString.Substring(_CurrentIndex)
        End Function

        ''' <summary>
        ''' Gibt die verbleibenden Zeichen bis zum Ende zurück.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Function ReadToEnd() As String
            Dim Temp = PeekToEnd()
            _CurrentIndex = _SourceString.Length
            Return Temp
        End Function

        ''' <summary>
        ''' Gibt die Zeichen bis zum nächsten vorkommen eines in <paramref name="Chars"/> vorkommenden End-Zeichens zurück, ohne sie zu entfernen.
        ''' Wird das Ende des Streams erreicht, bevor ein End-Zeichen gefunden wurde, wird der String bis zum Ende zurückgegeben.
        ''' Das gefundene End-Zeichen ist nicht im zurückgegebenen String enthalten.
        ''' </summary>
        ''' <param name="Chars">Die Zeichen, bis zu denen gelesen wird.</param>
        <DebuggerStepThrough()> _
        Public Function PeekToNext(ParamArray Chars As Char()) As String
            If Chars Is Nothing Then
                Throw New ArgumentNullException("Chars", "Chars darf nicht Nothing sein!")
            End If
            If Chars.Count = 0 Then
                Throw New ArgumentException("Chars muss mindestens ein Element beinhalten.", "Chars")
            End If
            EnsureIsInitialized()
            EnsureNotEndOfStream()
            Dim SB As New System.Text.StringBuilder
            Dim PeekOffset = _CurrentIndex
            Do
                Dim CurrentChar = _SourceString(PeekOffset)
                PeekOffset += 1
                If Chars.Contains(CurrentChar) Then
                    Exit Do
                Else
                    SB.Append(CurrentChar)
                End If
            Loop Until PeekOffset >= _SourceString.Length
            Return SB.ToString
        End Function

        ''' <summary>
        ''' Gibt die Zeichen bis zum nächsten vorkommen eines in <paramref name="Chars"/> vorkommenden End-Zeichens zurück.
        ''' Die Stream-Position wird hinter das gefundene End-Zeichen gesetzt.
        ''' Wird das Ende des Streams erreicht, bevor ein End-Zeichen gefunden wurde, wird der String bis zum Ende zurückgegeben.
        ''' Das gefundene End-Zeichen ist nicht im zurückgegebenen String enthalten.
        ''' </summary>
        ''' <param name="Chars">Die Zeichen, bis zu denen gelesen wird.</param>
        <DebuggerStepThrough()> _
        Public Function ReadToNext(ParamArray Chars As Char()) As String
            Dim Temp = PeekToNext(Chars)
            _CurrentIndex += Temp.Length + 1
            Return Temp
        End Function

        ''' <summary>
        ''' Gibt die Zeichen bis zum nächsten vorkommen eines in <paramref name="Strings"/> vorkommenden Strings zurück, ohne sie zu entfernen.
        ''' Wird das Ende des Streams erreicht, bevor ein End-Zeichen gefunden wurde, wird der String bis zum Ende zurückgegeben.
        ''' Der gefundene String aus <paramref name="Strings"/> ist nicht im zurückgegebenen String enthalten.
        ''' </summary>
        ''' <param name="Strings">Der String, bis zu denen gelesen wird.</param>
        <DebuggerStepThrough()> _
        Public Function PeekToNext(ParamArray Strings As String()) As String
            If Strings Is Nothing Then
                Throw New ArgumentNullException("Strings", "Strings darf nicht Nothing sein!")
            End If
            If Strings.Count = 0 Then
                Throw New ArgumentException("Strings muss mindestens ein Element beinhalten.", "Strings")
            End If
            EnsureIsInitialized()
            EnsureNotEndOfStream()
            Dim NextIndex As Integer? = Nothing
            For Each i In Strings
                Dim ThisIndex = _SourceString.IndexOf(i, _CurrentIndex)
                If Not ThisIndex = -1 Then
                    If Not NextIndex.HasValue OrElse ThisIndex < NextIndex.Value Then
                        NextIndex = ThisIndex
                    End If
                End If
            Next
            If NextIndex.HasValue Then
                Return _SourceString.SubstringBetween(_CurrentIndex, NextIndex.Value)
            End If
            Return _SourceString.Substring(_CurrentIndex)
        End Function

        ''' <summary>
        ''' Gibt die Zeichen bis zum nächsten vorkommen eines in <paramref name="Strings"/> vorkommenden End-Strings zurück.
        ''' Die Stream-Position wird hinter den End-String gesetzt.
        ''' Wird das Ende des Streams erreicht, bevor ein End-Zeichen gefunden wurde, wird der String bis zum Ende zurückgegeben.
        ''' Der gefundene End-String ist nicht im zurückgegebenen String enthalten.
        ''' </summary>
        ''' <param name="Strings">Der String, bis zu denen gelesen wird.</param>
        <DebuggerStepThrough()> _
        Public Function ReadToNext(ParamArray Strings As String()) As String
            If Strings Is Nothing Then
                Throw New ArgumentNullException("Strings", "Strings darf nicht Nothing sein!")
            End If
            If Strings.Count = 0 Then
                Throw New ArgumentException("Strings muss mindestens ein Element beinhalten.", "Strings")
            End If
            EnsureIsInitialized()
            EnsureNotEndOfStream()
            Dim NextIndex As Integer? = Nothing
            Dim EndString As String = Nothing
            For Each i In Strings
                Dim ThisIndex = _SourceString.IndexOf(i, _CurrentIndex)
                If Not ThisIndex = -1 Then
                    If Not NextIndex.HasValue OrElse ThisIndex < NextIndex.Value Then
                        NextIndex = ThisIndex
                        EndString = i
                    End If
                End If
            Next
            Dim TempCurrentIndex = _CurrentIndex
            If NextIndex.HasValue Then
                _CurrentIndex = NextIndex.Value + EndString.Length
                Return _SourceString.SubstringBetween(TempCurrentIndex, NextIndex.Value)
            End If
            _CurrentIndex = _SourceString.Length
            Return _SourceString.Substring(TempCurrentIndex)
        End Function

    End Class

End Namespace
