﻿
Imports System.ComponentModel

Namespace Wpf

    ''' <summary>
    ''' Stellt eine Basisklasse für ViewModels für WPF und MVVM dar.
    ''' Implementiert <see cref="INotifyPropertyChanged"/> und stellt Hilfsmethoden bereit, um damit zu arbeiten.
    ''' </summary>
    Public MustInherit Class ViewModelBase
        Implements INotifyPropertyChanged

        Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
        ''' <summary>
        ''' Löst für jedes Element in <paramref name="Names"/> das <see cref="INotifyPropertyChanged.PropertyChanged"/>-Event aus.
        ''' </summary>
        ''' <param name="Names">Die Namen der Properties, die sich geändert haben.</param>
        <DebuggerStepThrough()> _
        Protected Sub OnPropertyChanged(ParamArray Names As String())
            For Each i In Names
                RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(i))
            Next
        End Sub

        ''' <summary>
        ''' Prüft, ob <paramref name="Target"/> und <paramref name="NewValue"/> gleich sind (<see cref="System.Object.Equals"/>).
        ''' Ist das nicht der Fall, wird <paramref name="Target"/> überschrieben (wird ByRef übergeben), <see cref="OnPropertyChanged"/> aufgerufen und True zurückgegeben.
        ''' Andernfalls wird nur False zurückgegeben.
        ''' </summary>
        ''' <param name="Target">Üblicherweise ein Feld, das den Wert der Property speichert.</param>
        ''' <param name="NewValue">Üblicherweise der Parameter des Setters der Property</param>
        ''' <param name="PropertyNames">Die Namen der Eigenschaften, die sich geändert haben. Das können auch abhängige Eigenschaften sein (z.B. Kreisradius geändert -> Umfang ändert sich auch).</param>
        <DebuggerStepThrough()> _
        Protected Function ChangeIfDifferent(Of T)(ByRef Target As T, NewValue As T, ParamArray PropertyNames As String()) As Boolean
            If Object.Equals(Target, NewValue) Then
                Return False
            End If
            Target = NewValue
            OnPropertyChanged(PropertyNames)
            Return True
        End Function

        ''' <summary>
        ''' Prüft, ob <paramref name="Target"/> und <paramref name="NewValue"/> gleich sind (<see cref="System.Object.Equals"/>).
        ''' Ist das nicht der Fall, wird <paramref name="Target"/> überschrieben (wird ByRef übergeben) und True zurückgegeben.
        ''' Andernfalls wird nur False zurückgegeben.
        ''' </summary>
        ''' <param name="Target">Üblicherweise ein Feld, das den Wert der Property speichert.</param>
        ''' <param name="NewValue">Üblicherweise der Parameter des Setters der Property</param>
        <DebuggerStepThrough()> _
        Public Shared Function ChangeIfDifferent(Of T)(ByRef Target As T, NewValue As T) As Boolean
            If Object.Equals(Target, NewValue) Then
                Return False
            End If
            Target = NewValue
            Return True
        End Function

        ''' <summary>
        ''' Gibt an, ob der aktuelle Code im Designer von VisualStudio ausgeführt wird.
        ''' Kann verwendet werden, um z.B. Dummy-Daten im Designer anzuzeigen.
        ''' </summary>
        Public Shared ReadOnly Property IsInDesignMode As Boolean
            <DebuggerStepThrough()> _
            Get
                Return DesignerProperties.GetIsInDesignMode(New System.Windows.DependencyObject)
            End Get
        End Property

    End Class

End Namespace
